@@ -54,7 +54,7 @@ GQuark   tracker_data_error_quark                   (void);
 
 /* Metadata */
 void     tracker_data_delete_resource_description   (const gchar               *graph,
-                                                     const gchar               *uri,
+                                                     const gchar               *url,
                                                      GError                   **error);
 void     tracker_data_delete_statement              (const gchar               *graph,
                                                      const gchar               *subject,
