@@ -29,6 +29,7 @@
 #include <sys/types.h>
 #include <stdio.h>
 #include <fcntl.h>
+#include <math.h>
 
 #include <glib/gstdio.h>
 
@@ -53,7 +54,7 @@
 #define TRACKER_DB_PAGE_SIZE_DONT_SET -1
 
 /* Set current database version we are working with */
-#define TRACKER_DB_VERSION_NOW        TRACKER_DB_VERSION_8
+#define TRACKER_DB_VERSION_NOW        TRACKER_DB_VERSION_9
 #define TRACKER_DB_VERSION_FILE       "db-version.txt"
 
 #define IN_USE_FILENAME               ".meta.isrunning"
@@ -73,7 +74,8 @@ typedef enum {
 	TRACKER_DB_VERSION_5,       /* Version 0.7    (vstore branch) */
 	TRACKER_DB_VERSION_6,       /* Version 0.7.4  (nothing special) */
 	TRACKER_DB_VERSION_7,       /* Version 0.7.12 (nmo ontology) */
-	TRACKER_DB_VERSION_8        /* Version 0.7.13 (coalesce & writeback) */
+	TRACKER_DB_VERSION_8,       /* Version 0.7.13 (coalesce & writeback) */
+	TRACKER_DB_VERSION_9        /* Version 0.7.17 (mlo ontology) */
 } TrackerDBVersion;
 
 typedef struct {
@@ -305,6 +307,104 @@ tracker_db_manager_get_flags (void)
 	return old_flags;
 }
 
+/* Create a title-type string from the filename for replacing missing ones */
+static GValue
+function_sparql_string_from_filename (TrackerDBInterface *interface,
+				      gint                argc,
+				      GValue              values[])
+{
+	GValue  result = { 0, };
+	gchar  *name = NULL;
+	gchar  *suffix = NULL;
+
+	if (argc != 1) {
+		g_critical ("Invalid argument count");
+		return result;
+	}
+
+	/* "/home/user/path/title_of_the_movie.movie" -> "title of the movie"
+	   Only for local files currently, do we need to change? */
+
+	name = g_filename_display_basename (g_value_get_string (&values[0]));
+	if (!name) {
+		return result;
+	}
+
+	suffix = g_strrstr (name, ".");
+
+	if (suffix) {
+		*suffix = '\0';
+	}
+
+	g_strdelimit (name, "._", ' ');
+
+	g_value_init (&result, G_TYPE_STRING);
+	g_value_set_string (&result, name);
+
+	g_free (name);
+
+	return result;
+}
+
+static GValue
+function_sparql_cartesian_distance (TrackerDBInterface *interface,
+				    gint                argc,
+				    GValue              values[])
+{
+	GValue result = { 0, };
+
+	if (argc != 4) {
+		g_critical ("Invalid argument count");
+		return result;
+	}
+
+	gdouble lat1 = g_value_get_double (&values[0])*M_PI/180;
+	gdouble lat2 = g_value_get_double (&values[1])*M_PI/180;
+	gdouble lon1 = g_value_get_double (&values[2])*M_PI/180;
+	gdouble lon2 = g_value_get_double (&values[3])*M_PI/180;
+
+	gdouble R = 6371000;
+	gdouble a = M_PI/2 - lat1;
+	gdouble b = M_PI/2 - lat2;
+	gdouble c = sqrt(a*a + b*b - 2*a*b*cos(lon2 - lon1));
+	gdouble d = R*c;
+
+	g_value_init (&result, G_TYPE_DOUBLE);
+	g_value_set_double (&result, d);
+
+	return result;
+}
+
+static GValue
+function_sparql_haversine_distance (TrackerDBInterface *interface,
+				    gint                argc,
+				    GValue              values[])
+{
+	GValue result = { 0, };
+
+	if (argc != 4) {
+		g_critical ("Invalid argument count");
+		return result;
+	}
+
+	gdouble lat1 = g_value_get_double (&values[0])*M_PI/180;
+	gdouble lat2 = g_value_get_double (&values[1])*M_PI/180;
+	gdouble lon1 = g_value_get_double (&values[2])*M_PI/180;
+	gdouble lon2 = g_value_get_double (&values[3])*M_PI/180;
+
+	gdouble R = 6371000;
+	gdouble dLat = (lat2-lat1);
+	gdouble dLon = (lon2-lon1); 
+	gdouble a = sin(dLat/2) * sin(dLat/2) + cos(lat1) * cos(lat2) *  sin(dLon/2) * sin(dLon/2); 
+	gdouble c = 2 * atan2(sqrt(a), sqrt(1-a)); 
+	gdouble d = R * c;
+
+	g_value_init (&result, G_TYPE_DOUBLE);
+	g_value_set_double (&result, d);
+
+	return result;
+}
+
 static void
 function_group_concat_step (TrackerDBInterface *interface,
                             void               *aggregate_context,
@@ -694,7 +794,18 @@ db_set_params (TrackerDBInterface *iface,
 		                                             "SparqlRegex",
 		                                             function_sparql_regex,
 		                                             3);
-
+		tracker_db_interface_sqlite_create_function (iface,
+		                                             "SparqlStringFromFilename",
+		                                             function_sparql_string_from_filename,
+		                                             1);
+		tracker_db_interface_sqlite_create_function (iface,
+		                                             "SparqlCartesianDistance",
+		                                             function_sparql_cartesian_distance,
+		                                             4);
+		tracker_db_interface_sqlite_create_function (iface,
+		                                             "SparqlHaversineDistance",
+		                                             function_sparql_haversine_distance,
+		                                             4);
 		tracker_db_interface_sqlite_create_function (iface,
 		                                             "uncompress",
 		                                             function_uncompress,
@@ -872,17 +983,21 @@ db_manager_remove_all (gboolean rm_journal)
 	}
 
 	if (rm_journal) {
-		GFile *file;
-		gchar *cpath;
+		const gchar *opath = tracker_db_journal_get_filename ();
 
-		cpath = g_strdup (tracker_db_journal_get_filename ());
-		tracker_db_journal_shutdown ();
-		g_message ("  Removing journal:'%s'",
-		           cpath);
-		file = g_file_new_for_path (cpath);
-		g_file_delete (file, NULL, NULL);
-		g_object_unref (file);
-		g_free (cpath);
+		if (opath) {
+			GFile *file;
+			gchar *cpath;
+
+			cpath = g_strdup (opath);
+			tracker_db_journal_shutdown ();
+			g_message ("  Removing journal:'%s'",
+					   cpath);
+			file = g_file_new_for_path (cpath);
+			g_file_delete (file, NULL, NULL);
+			g_object_unref (file);
+			g_free (cpath);
+		}
 	}
 }
 
@@ -1271,9 +1386,7 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 		/* Load databases */
 		g_message ("Loading databases files...");
 
-		tracker_db_journal_reader_init (NULL);
 		must_recreate = !tracker_db_journal_reader_verify_last (NULL);
-		tracker_db_journal_reader_shutdown ();
 
 		if (!must_recreate && g_file_test (in_use_filename, G_FILE_TEST_EXISTS)) {
 			gsize size = 0;
