@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
  *
@@ -38,53 +37,76 @@ G_BEGIN_DECLS
 #define TRACKER_MINER_FS_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TRACKER_TYPE_MINER_FS, TrackerMinerFSClass))
 
 typedef struct TrackerMinerFS        TrackerMinerFS;
-typedef struct TrackerMinerFSClass   TrackerMinerFSClass;
 typedef struct TrackerMinerFSPrivate TrackerMinerFSPrivate;
 
-typedef void (* TrackerMinerFSDoneCb) (TrackerMinerFS       *fs,
-				       GFile                *file,
-				       TrackerSparqlBuilder *builder,
-				       const GError         *error,
-				       gpointer              user_data);
-
+/**
+ * TrackerMinerFS:
+ *
+ * Abstract miner abstract implementation to get data
+ * from the filesystem.
+ **/
 struct TrackerMinerFS {
 	TrackerMiner parent;
 	TrackerMinerFSPrivate *private;
 };
 
-struct TrackerMinerFSClass {
+/**
+ * TrackerMinerFSClass:
+ * @parent: parent object class
+ * @check_file: Called when a file should be checked for further processing
+ * @check_directory: Called when a directory should be checked for further processing
+ * @check_directory_contents: Called when a directory should be checked for further processing, based on the directory contents.
+ * @process_file: Called when the metadata associated to a file is requested.
+ * @writeback_file: Called after a writeback event happens on a file.
+ * @monitor_directory: Called to check whether a directory should be modified.
+ * @finished: Called when all processing has been performed.
+ *
+ * Prototype for the abstract class, @check_file, @check_directory, @check_directory_contents,
+ * @process_file and @monitor_directory must be implemented in the deriving class in order to
+ * actually extract data.
+ **/
+typedef struct {
 	TrackerMinerClass parent;
 
 	gboolean (* check_file)            (TrackerMinerFS       *fs,
-					    GFile                *file);
+	                                    GFile                *file);
 	gboolean (* check_directory)       (TrackerMinerFS       *fs,
-					    GFile                *file);
+	                                    GFile                *file);
 	gboolean (* check_directory_contents) (TrackerMinerFS    *fs,
-					       GFile             *parent,
-					       GList             *children);
+	                                       GFile             *parent,
+	                                       GList             *children);
 	gboolean (* process_file)          (TrackerMinerFS       *fs,
-					    GFile                *file,
-					    TrackerSparqlBuilder *builder,
-					    GCancellable         *cancellable,
-					    TrackerMinerFSDoneCb  done_cb,
-					    gpointer              done_cb_data);
+	                                    GFile                *file,
+	                                    TrackerSparqlBuilder *builder,
+	                                    GCancellable         *cancellable);
+	gboolean (* writeback_file)        (TrackerMinerFS       *fs,
+	                                    GFile                *file,
+	                                    TrackerSparqlBuilder *builder,
+	                                    GCancellable         *cancellable);
+
 	gboolean (* monitor_directory)     (TrackerMinerFS       *fs,
-					    GFile                *file);
+	                                    GFile                *file);
 	void     (* finished)              (TrackerMinerFS       *fs);
-};
+} TrackerMinerFSClass;
 
 GType    tracker_miner_fs_get_type         (void) G_GNUC_CONST;
 
 void     tracker_miner_fs_add_directory    (TrackerMinerFS *fs,
-					    GFile          *file,
-					    gboolean        recurse);
+                                            GFile          *file,
+                                            gboolean        recurse);
 gboolean tracker_miner_fs_remove_directory (TrackerMinerFS *fs,
-					    GFile          *file);
+                                            GFile          *file);
 
 void     tracker_miner_fs_set_throttle     (TrackerMinerFS *fs,
-					    gdouble         throttle);
+                                            gdouble         throttle);
 gdouble  tracker_miner_fs_get_throttle     (TrackerMinerFS *fs);
 
+void     tracker_miner_fs_notify_file      (TrackerMinerFS *fs,
+                                            GFile          *file,
+                                            const GError   *error);
+void     tracker_miner_fs_add_file         (TrackerMinerFS *fs,
+                                            GFile          *file);
+
 G_END_DECLS
 
 #endif /* __LIBTRACKERMINER_MINER_FS_H__ */
