@@ -32,9 +32,8 @@
 #include <gst/gst.h>
 #include <gst/tag/tag.h>
 
-#include <libtracker-common/tracker-statement-list.h>
+#include <libtracker-extract/tracker-extract.h>
 
-#include "tracker-main.h"
 #include "tracker-albumart.h"
 
 typedef enum {
@@ -69,8 +68,12 @@ typedef struct {
 
 } MetadataExtractor;
 
-static void extract_gstreamer_helix_audio (const gchar *uri, TrackerSparqlBuilder *metadata);
-static void extract_gstreamer_helix_video (const gchar *uri, TrackerSparqlBuilder *metadata);
+static void extract_gstreamer_helix_audio (const gchar          *uri,
+                                           TrackerSparqlBuilder *preupdate,
+                                           TrackerSparqlBuilder *metadata);
+static void extract_gstreamer_helix_video (const gchar          *uri,
+                                           TrackerSparqlBuilder *preupdate,
+                                           TrackerSparqlBuilder *metadata);
 
 static TrackerExtractData data[] = {
 	{ "audio/vnd.rn-realaudio", extract_gstreamer_helix_audio },
@@ -764,7 +767,8 @@ poll_for_state_change (MetadataExtractor *extractor,
 
 static void
 tracker_extract_gstreamer_helix (const gchar *uri,
-                                 TrackerSparqlBuilder  *metadata,
+                                 TrackerSparqlBuilder *preupdate,
+                                 TrackerSparqlBuilder *metadata,
                                  ExtractMime    type)
 {
 	MetadataExtractor *extractor;
@@ -844,19 +848,23 @@ tracker_extract_gstreamer_helix (const gchar *uri,
 }
 
 static void
-extract_gstreamer_helix_audio (const gchar *uri, TrackerSparqlBuilder *metadata)
+extract_gstreamer_helix_audio (const gchar          *uri,
+                               TrackerSparqlBuilder *preupdate,
+			       TrackerSparqlBuilder *metadata)
 {
-	tracker_extract_gstreamer_helix (uri, metadata, EXTRACT_MIME_AUDIO);
+	tracker_extract_gstreamer_helix (uri, preupdate, metadata, EXTRACT_MIME_AUDIO);
 }
 
 static void
-extract_gstreamer_helix_video (const gchar *uri, TrackerSparqlBuilder *metadata)
+extract_gstreamer_helix_video (const gchar          *uri,
+                               TrackerSparqlBuilder *preupdate,
+			       TrackerSparqlBuilder *metadata)
 {
-	tracker_extract_gstreamer_helix (uri, metadata, EXTRACT_MIME_VIDEO);
+	tracker_extract_gstreamer_helix (uri, preupdate, metadata, EXTRACT_MIME_VIDEO);
 }
 
 TrackerExtractData *
-tracker_get_extract_data (void)
+tracker_extract_get_data (void)
 {
 	return data;
 }
