@@ -45,6 +45,7 @@ typedef struct {
 } PDFData;
 
 static void extract_pdf (const gchar          *uri,
+                         TrackerSparqlBuilder *preupdate,
                          TrackerSparqlBuilder *metadata);
 
 static TrackerExtractData data[] = {
@@ -120,6 +121,7 @@ extract_content (PopplerDocument *document,
 
 		g_free (text);
 		g_free (t);
+		g_object_unref (page);
 	}
 
 	return g_string_free (string, FALSE);
@@ -166,6 +168,7 @@ write_pdf_data (PDFData               data,
 
 static void
 extract_pdf (const gchar          *uri,
+             TrackerSparqlBuilder *preupdate,
              TrackerSparqlBuilder *metadata)
 {
 	TrackerFTSConfig *fts_config;
@@ -199,7 +202,6 @@ extract_pdf (const gchar          *uri,
 		return;
 	}
 
-	tracker_sparql_builder_subject_iri (metadata, uri);
 	tracker_sparql_builder_predicate (metadata, "a");
 	tracker_sparql_builder_object (metadata, "nfo:PaginatedTextDocument");
 
@@ -439,6 +441,9 @@ extract_pdf (const gchar          *uri,
 		
 			tracker_sparql_builder_object_blank_close (metadata);
 		}
+
+		/* PDF keywords aren't used ATM */
+		g_free (pd.keywords);
 	} else {
 		/* So if we are here we have NO XMP data and we just
 		 * write what we know from Poppler.
