@@ -70,6 +70,8 @@ struct TrackerMinerFilesPrivate {
 	guint low_battery_pause_cookie;
 
 	DBusGProxy *extractor_proxy;
+
+	GQuark quark_mount_point_udi;
 };
 
 enum {
@@ -209,6 +211,8 @@ tracker_miner_files_init (TrackerMinerFiles *mf)
 	/* Set up extractor and signals */
 	priv->extractor_proxy = extractor_create_proxy ();
 
+	priv->quark_mount_point_udi = g_quark_from_static_string ("tracker-mount-point-udi");
+
 	init_mount_points (mf);
 }
 
@@ -396,6 +400,14 @@ miner_files_constructed (GObject *object)
 
 	for (m = mounts; m; m = m->next) {
 		GFile *file = g_file_new_for_path (m->data);
+#ifdef HAVE_HAL
+		const gchar *udi = tracker_storage_get_volume_udi_for_file (mf->private->storage, file);
+
+		g_object_set_qdata_full (G_OBJECT (file),
+					 mf->private->quark_mount_point_udi,
+					 g_strdup (udi),
+					 (GDestroyNotify) g_free);
+#endif
 
 		g_message ("  Adding:'%s'", (gchar*) m->data);
 		tracker_miner_fs_add_directory (TRACKER_MINER_FS (mf),
@@ -457,9 +469,15 @@ set_up_mount_point (TrackerMinerFiles *miner,
 			uri = g_file_get_uri (file);
 
 			g_string_append_printf (queries,
-			                        "DROP GRAPH <%s>\n "
-			                        "INSERT INTO <%s> { <%s> a tracker:Volume; tracker:mountPoint <%s> } ",
-			                        removable_device_urn, removable_device_urn, removable_device_urn, uri);
+			                        "DROP GRAPH <%s> "
+			                        "INSERT INTO <%s> { "
+			                        "  <%s> a tracker:Volume; "
+			                        "       tracker:mountPoint ?u "
+			                        "} WHERE { "
+			                        "  ?u a nfo:FileDataObject; "
+			                        "     nie:url \"%s\" "
+			                        "}",
+						removable_device_urn, removable_device_urn, removable_device_urn, uri);
 
 			g_object_unref (file);
 			g_free (uri);
@@ -709,6 +727,11 @@ mount_point_added_cb (TrackerStorage *storage,
 		GFile *file;
 
 		file = g_file_new_for_path (mount_point);
+		g_object_set_qdata_full (G_OBJECT (file),
+		                         priv->quark_mount_point_udi,
+		                         g_strdup (udi),
+		                         (GDestroyNotify) g_free);
+
 		tracker_miner_fs_add_directory (TRACKER_MINER_FS (user_data),
 		                                file,
 		                                TRUE);
@@ -1169,7 +1192,7 @@ miner_files_add_to_datasource (TrackerMinerFiles    *mf,
 		removable_device_urn = g_strdup (TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN);
 	}
 
-	tracker_sparql_builder_subject_iri (sparql, uri);
+	tracker_sparql_builder_subject (sparql, "_:file");
 	tracker_sparql_builder_predicate (sparql, "a");
 	tracker_sparql_builder_object (sparql, "nfo:FileDataObject");
 
@@ -1224,11 +1247,13 @@ extractor_create_proxy (void)
 
 static void
 extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
+                                    gchar      *preupdate,
                                     gchar      *sparql,
                                     GError     *error,
                                     gpointer    user_data)
 {
 	ProcessFileData *data = user_data;
+	const gchar *udi;
 
 	if (error) {
 		/* Something bad happened, notify about the error */
@@ -1238,14 +1263,61 @@ extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
 		return;
 	}
 
-	if (sparql) {
+	if (sparql && *sparql) {
+		tracker_sparql_builder_append (data->sparql, "\n");
 		tracker_sparql_builder_append (data->sparql, sparql);
-		g_free (sparql);
+	}
+
+	tracker_sparql_builder_insert_close (data->sparql);
+
+	/* Prepend preupdate queries */
+	if (preupdate && *preupdate) {
+		tracker_sparql_builder_prepend (data->sparql, preupdate);
+	}
+
+	udi = g_object_get_qdata (G_OBJECT (data->file),
+				  data->miner->private->quark_mount_point_udi);
+
+	/* File represents a mount point */
+	if (G_UNLIKELY (udi)) {
+		GString *queries;
+		gchar *removable_device_urn, *uri;
+
+		removable_device_urn = g_strdup_printf (TRACKER_DATASOURCE_URN_PREFIX "%s", udi);
+		uri = g_file_get_uri (G_FILE (data->file));
+
+		queries = g_string_new ("");
+		g_string_append_printf (queries,
+		                        "DELETE FROM <%s> { "
+		                        "  <%s> tracker:mountPoint ?unknown "
+		                        "} WHERE { "
+		                        "  <%s> a tracker:Volume; "
+		                        "       tracker:mountPoint ?unknown "
+		                        "} ",
+		                        removable_device_urn, removable_device_urn, removable_device_urn);
+
+		g_string_append_printf (queries,
+		                        "INSERT INTO <%s> { "
+		                        "  <%s> a tracker:Volume; "
+		                        "       tracker:mountPoint ?u "
+		                        "} WHERE { "
+		                        "  ?u a nfo:FileDataObject; "
+		                        "     nie:url \"%s\" "
+		                        "}",
+		                        removable_device_urn, removable_device_urn, uri);
+
+		tracker_sparql_builder_append (data->sparql, queries->str);
+		g_string_free (queries, TRUE);
+		g_free (removable_device_urn);
+		g_free (uri);
 	}
 
 	/* Notify about the success */
 	tracker_miner_fs_notify_file (TRACKER_MINER_FS (data->miner), data->file, NULL);
+
 	process_file_data_free (data);
+	g_free (preupdate);
+	g_free (sparql);
 }
 
 static void
@@ -1311,9 +1383,15 @@ process_file_cb (GObject      *object,
 
 	tracker_sparql_builder_insert_open (sparql, uri);
 
-	tracker_sparql_builder_subject_iri (sparql, uri);
+	tracker_sparql_builder_subject (sparql, "_:file");
+
 	tracker_sparql_builder_predicate (sparql, "a");
 	tracker_sparql_builder_object (sparql, "nfo:FileDataObject");
+	tracker_sparql_builder_object (sparql, "nie:InformationElement");
+
+	if (g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY) {
+		tracker_sparql_builder_object (sparql, "nfo:Folder");
+	}
 
 	parent = g_file_get_parent (file);
 	if (parent) {
@@ -1338,17 +1416,9 @@ process_file_cb (GObject      *object,
 	tracker_sparql_builder_predicate (sparql, "nfo:fileLastAccessed");
 	tracker_sparql_builder_object_date (sparql, (time_t *) &time_);
 
-	tracker_sparql_builder_subject_iri (sparql, uri); /* Change to URN */
-	tracker_sparql_builder_predicate (sparql, "a");
-	tracker_sparql_builder_object (sparql, "nie:InformationElement");
-
-	if (g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY) {
-		tracker_sparql_builder_object (sparql, "nfo:Folder");
-	}
-
 	/* Laying the link between the IE and the DO. We use IE = DO */
 	tracker_sparql_builder_predicate (sparql, "nie:isStoredAs");
-	tracker_sparql_builder_object_iri (sparql, uri);
+	tracker_sparql_builder_object (sparql, "_:file");
 
 	/* The URL of the DataObject (because IE = DO, this is correct) */
 	tracker_sparql_builder_predicate (sparql, "nie:url");
@@ -1359,8 +1429,6 @@ process_file_cb (GObject      *object,
 
 	miner_files_add_to_datasource (data->miner, file, sparql);
 
-	tracker_sparql_builder_insert_close (sparql);
-
 	/* Next step, getting embedded metadata */
 	extractor_get_embedded_metadata (data, uri, mime_type);
 
@@ -1436,10 +1504,9 @@ miner_files_writeback_file (TrackerMinerFS       *fs,
 
 	/* For writeback we only write a few properties back. These properties
 	 * should NEVER be marked as tracker:writeback in the ontology ! */
-
 	tracker_sparql_builder_insert_open (sparql, uri);
 
-	tracker_sparql_builder_subject_iri (sparql, uri);
+	tracker_sparql_builder_subject_variable (sparql, "urn");
 	tracker_sparql_builder_predicate (sparql, "a");
 	tracker_sparql_builder_object (sparql, "nfo:FileDataObject");
 
@@ -1457,11 +1524,19 @@ miner_files_writeback_file (TrackerMinerFS       *fs,
 	tracker_sparql_builder_predicate (sparql, "nie:mimeType");
 	tracker_sparql_builder_object_string (sparql, mime_type);
 
+	tracker_sparql_builder_insert_close (sparql);
+
+	tracker_sparql_builder_where_open (sparql);
+
+	tracker_sparql_builder_subject_variable (sparql, "urn");
+	tracker_sparql_builder_predicate (sparql, "nie:url");
+	tracker_sparql_builder_object_string (sparql, uri);
+
+	tracker_sparql_builder_where_close (sparql);
+
 	g_object_unref (file_info);
 	g_free (uri);
 
-	tracker_sparql_builder_insert_close (sparql);
-
 	return TRUE;
 }
 
